@@ -13,6 +13,7 @@ from moving_targets.masters.losses import MAE, MSE, Loss, HammingDistance
 from moving_targets.masters.optimizers import Constant, Optimizer
 from moving_targets.util import probabilities
 from moving_targets.util.errors import not_implemented_message
+from moving_targets.util.masking import get_mask
 from moving_targets.util.scalers import Scaler
 from moving_targets.util.typing import Dataset
 
@@ -28,6 +29,7 @@ class Master(StatsLogger):
                  backend: Union[str, Backend],
                  loss: Union[str, Loss],
                  alpha: Union[str, float, Optimizer],
+                 mask: Optional[float] = None,
                  x_scaler: Union[None, Scaler, str] = None,
                  y_scaler: Union[None, Scaler, str] = None,
                  stats: Union[bool, List[str]] = False):
@@ -42,6 +44,9 @@ class Master(StatsLogger):
             Either a floating point for a constant alpha value, a string representing an `Optimizer` alias,  or an
             actual `Optimizer` instance which implements the strategy to dynamically change the alpha value.
 
+        :param mask:
+            The (optional) masking value used to mask the original targets.
+
         :param x_scaler:
             The (optional) scaler for the input data, or a string representing the default scaling method.
 
@@ -70,6 +75,9 @@ class Master(StatsLogger):
         self.alpha: Optimizer = alpha
         """The alpha `Optimizer` instance."""
 
+        self.mask: Optional[float] = mask
+        """The (optional) masking value used to mask the original targets."""
+
         self.x_scaler: Optional[Scaler] = Scaler(default_method=x_scaler) if isinstance(x_scaler, str) else x_scaler
         """The (optional) scaler for the input data."""
 
@@ -135,6 +143,23 @@ class Master(StatsLogger):
         self._time = None
         self._macs = None
 
+    def on_backend_solved(self, x, y: np.ndarray, p: np.ndarray, z: np.ndarray):
+        """Routine called after the master problem is solved and the adjusted targets are retrieved.
+
+        :param x:
+            The matrix/dataframe of training samples.
+
+        :param y:
+            The vector of training labels.
+
+        :param p:
+            The vector of learners predictions.
+
+        :param z:
+            The vector of adjusted labels.
+        """
+        pass
+
     def build(self, x, y: np.ndarray, p: np.ndarray) -> np.ndarray:
         """Creates the model variables and adds the problem constraints.
 
@@ -179,7 +204,12 @@ class Master(StatsLogger):
         # transform data if scalers are passed
         x = x if self.x_scaler is None else self.x_scaler.fit_transform(data=x)
         if self.y_scaler is not None:
-            y = self.y_scaler.fit_transform(data=y)
+            # this is a sort of workaround: the masking of the vectors is delegated to the loss instance, thus there
+            # should be no need to mask the vector at this point, however, since here we want to scale the target data
+            # which might be masked later, we fit the scaler only on the masked data (this will avoid numerical errors
+            # in case of nan masking, or wrong scaling factors in case of value-based or vector-based masking)
+            self.y_scaler.fit(y[get_mask(y, self.mask)])
+            y = self.y_scaler.transform(data=y)
             p = self.y_scaler.transform(data=p)
         # build backend and model
         self.backend.build()
@@ -189,7 +219,8 @@ class Master(StatsLogger):
                                              variables=v,
                                              targets=y,
                                              predictions=p,
-                                             sample_weight=sample_weight)
+                                             sample_weight=sample_weight,
+                                             mask=self.mask)
         self.backend.minimize(cost=alpha * nabla_term + squared_term)
         # if the problem is infeasible return None, otherwise log stats and return the adjusted labels
         if self.backend.solve().solution is not None:
@@ -201,6 +232,7 @@ class Master(StatsLogger):
         else:
             self._log_stats(alpha=alpha)
             adjusted = None
+        self.on_backend_solved(x=x, y=y, p=p, z=adjusted)
         self.backend.clear()
         # invert transformation on output targets if scaler is passed
         return adjusted if self.y_scaler is None else self.y_scaler.inverse_transform(adjusted)
@@ -214,6 +246,7 @@ class RegressionMaster(Master, ABC):
                  alpha: Union[str, float, Optimizer] = 'harmonic',
                  lb: float = -float('inf'),
                  ub: float = float('inf'),
+                 mask: Optional[float] = None,
                  x_scaler: Union[None, Scaler, str] = None,
                  y_scaler: Union[None, Scaler, str] = None,
                  stats: Union[bool, List[str]] = False):
@@ -234,6 +267,9 @@ class RegressionMaster(Master, ABC):
         :param ub:
             The model variables upper bounds.
 
+        :param mask:
+            The (optional) masking value used to mask the original targets.
+
         :param x_scaler:
             The (optional) scaler for the input data, or a string representing the default scaling method.
 
@@ -247,6 +283,7 @@ class RegressionMaster(Master, ABC):
         super(RegressionMaster, self).__init__(backend=backend,
                                                loss=loss,
                                                alpha=alpha,
+                                               mask=mask,
                                                x_scaler=x_scaler,
                                                y_scaler=y_scaler,
                                                stats=stats)
@@ -276,6 +313,7 @@ class ClassificationMaster(Master, ABC):
                  alpha: Union[str, float, Optimizer] = 'harmonic',
                  labelling: bool = False,
                  types: str = 'auto',
+                 mask: Optional[float] = None,
                  x_scaler: Union[None, Scaler, str] = None,
                  y_scaler: Union[None, Scaler, str] = None,
                  stats: Union[bool, List[str]] = False):
@@ -308,6 +346,9 @@ class ClassificationMaster(Master, ABC):
             passed, since in that case it will leverage the 'binary' field of the loss to choose the variables types
             while returning discrete adjustments.
 
+        :param mask:
+            The (optional) masking value used to mask the original targets.
+
         :param x_scaler:
             The (optional) scaler for the input data, or a string representing the default scaling method.
 
@@ -341,6 +382,7 @@ class ClassificationMaster(Master, ABC):
         super(ClassificationMaster, self).__init__(backend=backend,
                                                    loss=loss,
                                                    alpha=alpha,
+                                                   mask=mask,
                                                    x_scaler=x_scaler,
                                                    y_scaler=y_scaler,
                                                    stats=stats)
